@@ -1,31 +1,226 @@
 package Net::DNS::RR::DNSKEY;
 
-#
-# $Id: DNSKEY.pm 1781 2020-05-13 08:58:25Z willem $
-#
-our $VERSION = (qw$LastChangedRevision: 1781 $)[1];
-
-
 use strict;
 use warnings;
+our $VERSION = (qw$Id: DNSKEY.pm 1910 2023-03-30 19:16:30Z willem $)[2];
+
 use base qw(Net::DNS::RR);
 
+
 =head1 NAME
 
 Net::DNS::RR::DNSKEY - DNS DNSKEY resource record
 
 =cut
 
-
 use integer;
 
 use Carp;
 
-use constant BASE64 => defined eval 'require MIME::Base64';
+use constant BASE64 => defined eval { require MIME::Base64 };
+
+
+sub _decode_rdata {			## decode rdata from wire-format octet string
+	my ( $self, $data, $offset ) = @_;
+
+	my $rdata = substr $$data, $offset, $self->{rdlength};
+	@{$self}{qw(flags protocol algorithm keybin)} = unpack 'n C2 a*', $rdata;
+	return;
+}
+
+
+sub _encode_rdata {			## encode rdata as wire-format octet string
+	my $self = shift;
+
+	return pack 'n C2 a*', @{$self}{qw(flags protocol algorithm keybin)};
+}
+
+
+sub _format_rdata {			## format rdata portion of RR string.
+	my $self = shift;
+
+	my @rdata = @{$self}{qw(flags protocol algorithm)};
+	if ( my $keybin = $self->keybin ) {
+		$self->_annotation( 'Key ID =', $self->keytag );
+		return $self->SUPER::_format_rdata() unless BASE64;
+		push @rdata, split /\s+/, MIME::Base64::encode($keybin);
+	} else {
+		push @rdata, '""';
+	}
+	return @rdata;
+}
+
+
+sub _parse_rdata {			## populate RR from rdata in argument list
+	my ( $self, @argument ) = @_;
+
+	$self->flags( shift @argument );
+	$self->protocol( shift @argument );
+	my $algorithm = shift @argument;
+	$self->key(@argument) if $algorithm;
+	$self->algorithm($algorithm);
+	return;
+}
+
+
+sub _defaults {				## specify RR attribute default values
+	my $self = shift;
+
+	$self->flags(256);
+	$self->protocol(3);
+	$self->algorithm(1);
+	$self->keybin('');
+	return;
+}
+
+
+sub flags {
+	my ( $self, @value ) = @_;
+	for (@value) { $self->{flags} = 0 + $_ }
+	return $self->{flags} || 0;
+}
+
+
+sub zone {
+	my ( $self, @value ) = @_;
+	for ( $self->{flags} |= 0 ) {
+		if ( scalar @value ) {
+			$_ |= 0x0100;
+			$_ ^= 0x0100 unless shift @value;
+		}
+	}
+	return $self->{flags} & 0x0100;
+}
+
+
+sub revoke {
+	my ( $self, @value ) = @_;
+	for ( $self->{flags} |= 0 ) {
+		if ( scalar @value ) {
+			$_ |= 0x0080;
+			$_ ^= 0x0080 unless shift @value;
+		}
+	}
+	return $self->{flags} & 0x0080;
+}
+
+
+sub sep {
+	my ( $self, @value ) = @_;
+	for ( $self->{flags} |= 0 ) {
+		if ( scalar @value ) {
+			$_ |= 0x0001;
+			$_ ^= 0x0001 unless shift @value;
+		}
+	}
+	return $self->{flags} & 0x0001;
+}
+
+
+sub protocol {
+	my ( $self, @value ) = @_;
+	for (@value) { $self->{protocol} = 0 + $_ }
+	return $self->{protocol} || 0;
+}
+
+
+sub algorithm {
+	my ( $self, $arg ) = @_;
+
+	unless ( ref($self) ) {		## class method or simple function
+		my $argn = pop;
+		return $argn =~ /\D/ ? _algbyname($argn) : _algbyval($argn);
+	}
+
+	return $self->{algorithm} unless defined $arg;
+	return _algbyval( $self->{algorithm} ) if uc($arg) eq 'MNEMONIC';
+	return $self->{algorithm} = _algbyname($arg) || die _algbyname('')    # disallow algorithm(0)
+}
+
+
+sub key {
+	my ( $self, @value ) = @_;
+	return MIME::Base64::encode( $self->keybin(), "" ) unless scalar @value;
+	return $self->keybin( MIME::Base64::decode( join "", @value ) );
+}
+
+
+sub keybin {
+	my ( $self, @value ) = @_;
+	for (@value) { $self->{keybin} = $_ }
+	return $self->{keybin} || "";
+}
+
+
+sub publickey {
+	my ( $self, @value ) = @_;
+	return $self->key(@value);
+}
+
+
+sub privatekeyname {
+	my $self = shift;
+	my $name = $self->signame;
+	return sprintf 'K%s+%03d+%05d.private', $name, $self->algorithm, $self->keytag;
+}
+
+
+sub signame {
+	my $self = shift;
+	return lc $self->{owner}->fqdn;
+}
+
+
+sub keylength {
+	my $self = shift;
+
+	my $keybin = $self->keybin || return;
+
+	local $_ = _algbyval( $self->{algorithm} );
+
+	if (/^RSA/) {
+
+		# Modulus length, see RFC 3110
+		if ( my $exp_length = unpack 'C', $keybin ) {
+
+			return ( length($keybin) - $exp_length - 1 ) << 3;
+
+		} else {
+			$exp_length = unpack 'x n', $keybin;
+			return ( length($keybin) - $exp_length - 3 ) << 3;
+		}
+
+	} elsif (/^DSA/) {
+
+		# Modulus length, see RFC 2536
+		my $T = unpack 'C', $keybin;
+		return ( $T << 6 ) + 512;
+	}
+
+	return length($keybin) << 2;	## ECDSA / EdDSA
+}
+
+
+sub keytag {
+	my $self = shift;
+
+	my $keybin = $self->{keybin} || return;
+
+	# RFC4034 Appendix B.1: most significant 16 bits of least significant 24 bits
+	return unpack 'n', substr $keybin, -3 if $self->{algorithm} == 1;
+
+	# RFC4034 Appendix B
+	my $od = length($keybin) & 1;
+	my $rd = pack "n C2 a* x$od", @{$self}{qw(flags protocol algorithm)}, $keybin;
+	my $ac = 0;
+	$ac += $_ for unpack 'n*', $rd;
+	$ac += ( $ac >> 16 );
+	return $ac & 0xFFFF;
+}
+
+
+########################################
 
-#
-# source: http://www.iana.org/assignments/dns-sec-alg-numbers
-#
 {
 	my @algbyname = (
 		'DELETE'	     => 0,			# [RFC4034][RFC4398][RFC8078]
@@ -54,7 +249,8 @@ use constant BASE64 => defined eval 'require MIME::Base64';
 
 	my %algbyval = reverse @algbyname;
 
-	my @algrehash = map /^\d/ ? ($_) x 3 : do { s/[\W_]//g; uc($_) }, @algbyname;
+	foreach (@algbyname) { s/[\W_]//g; }			# strip non-alphanumerics
+	my @algrehash = map { /^\d/ ? ($_) x 3 : uc($_) } @algbyname;
 	my %algbyname = @algrehash;				# work around broken cperl
 
 	sub _algbyname {
@@ -63,196 +259,16 @@ use constant BASE64 => defined eval 'require MIME::Base64';
 		$key =~ s/[\W_]//g;				# strip non-alphanumerics
 		my $val = $algbyname{$key};
 		return $val if defined $val;
-		return $key =~ /^\d/ ? $arg : croak qq[unknown algorithm "$arg"];
+		return $key =~ /^\d/ ? $arg : croak qq[unknown algorithm $arg];
 	}
 
 	sub _algbyval {
 		my $value = shift;
-		$algbyval{$value} || return $value;
+		return $algbyval{$value} || return $value;
 	}
 }
 
-
-sub _decode_rdata {			## decode rdata from wire-format octet string
-	my $self = shift;
-	my ( $data, $offset ) = @_;
-
-	my $rdata = substr $$data, $offset, $self->{rdlength};
-	$self->{keybin} = unpack '@4 a*', $rdata;
-	@{$self}{qw(flags protocol algorithm)} = unpack 'n C*', $rdata;
-}
-
-
-sub _encode_rdata {			## encode rdata as wire-format octet string
-	my $self = shift;
-
-	pack 'n C2 a*', @{$self}{qw(flags protocol algorithm keybin)};
-}
-
-
-sub _format_rdata {			## format rdata portion of RR string.
-	my $self = shift;
-
-	my $algorithm = $self->{algorithm};
-	$self->_annotation( 'Key ID =', $self->keytag ) if $algorithm;
-	return $self->SUPER::_format_rdata() unless BASE64;
-	my @param = ( @{$self}{qw(flags protocol)}, $algorithm );
-	my @rdata = ( @param, split /\s+/, MIME::Base64::encode( $self->{keybin} ) || '-' );
-}
-
-
-sub _parse_rdata {			## populate RR from rdata in argument list
-	my $self = shift;
-
-	my $flags = shift;		## avoid destruction by CDNSKEY algorithm(0)
-	$self->protocol(shift);
-	$self->algorithm(shift);
-	$self->flags($flags);
-	$self->key(@_);
-}
-
-
-sub _defaults {				## specify RR attribute default values
-	my $self = shift;
-
-	$self->algorithm(1);
-	$self->flags(256);
-	$self->protocol(3);
-	$self->keybin('');
-}
-
-
-sub flags {
-	my $self = shift;
-
-	$self->{flags} = 0 + shift if scalar @_;
-	$self->{flags} || 0;
-}
-
-
-sub zone {
-	for ( shift->{flags} ) {
-		$_ = ( $_[0] ? 0 : 0x0100 ) ^ ( 0x0100 | ( $_ || 0 ) ) if scalar @_;
-		return 0x0100 & ( $_ || 0 );
-	}
-}
-
-
-sub revoke {
-	for ( shift->{flags} ) {
-		$_ = ( $_[0] ? 0 : 0x0080 ) ^ ( 0x0080 | ( $_ || 0 ) ) if scalar @_;
-		return 0x0080 & ( $_ || 0 );
-	}
-}
-
-
-sub sep {
-	for ( shift->{flags} ) {
-		$_ = ( $_[0] ? 0 : 0x0001 ) ^ ( 0x0001 | ( $_ || 0 ) ) if scalar @_;
-		return 0x0001 & ( $_ || 0 );
-	}
-}
-
-
-sub protocol {
-	my $self = shift;
-
-	$self->{protocol} = 0 + shift if scalar @_;
-	$self->{protocol} || 0;
-}
-
-
-sub algorithm {
-	my ( $self, $arg ) = @_;
-
-	unless ( ref($self) ) {		## class method or simple function
-		my $argn = pop;
-		return $argn =~ /\D/ ? _algbyname($argn) : _algbyval($argn);
-	}
-
-	return $self->{algorithm} unless defined $arg;
-	return _algbyval( $self->{algorithm} ) if uc($arg) eq 'MNEMONIC';
-	$self->{algorithm} = _algbyname($arg) || die _algbyname('')    # disallow algorithm(0)
-}
-
-
-sub key {
-	my $self = shift;
-	return MIME::Base64::encode( $self->keybin(), "" ) unless scalar @_;
-	$self->keybin( MIME::Base64::decode( join "", @_ ) );
-}
-
-
-sub keybin {
-	my $self = shift;
-
-	$self->{keybin} = shift if scalar @_;
-	$self->{keybin} || "";
-}
-
-
-sub publickey { shift->key(@_); }
-
-
-sub privatekeyname {
-	my $self = shift;
-	my $name = $self->signame;
-	sprintf 'K%s+%03d+%05d.private', $name, $self->algorithm, $self->keytag;
-}
-
-
-sub signame {
-	my $self = shift;
-	my $name = lc $self->{owner}->fqdn;
-}
-
-
-sub keylength {
-	my $self = shift;
-
-	my $keybin = $self->keybin || return undef;
-
-	local $_ = _algbyval( $self->{algorithm} );
-
-	if (/^RSA/) {
-
-		# Modulus length, see RFC 3110
-		if ( my $exp_length = unpack 'C', $keybin ) {
-
-			return ( length($keybin) - $exp_length - 1 ) << 3;
-
-		} else {
-			$exp_length = unpack 'x n', $keybin;
-			return ( length($keybin) - $exp_length - 3 ) << 3;
-		}
-
-	} elsif (/^DSA/) {
-
-		# Modulus length, see RFC 2536
-		my $T = unpack 'C', $keybin;
-		return ( $T << 6 ) + 512;
-	}
-
-	length($keybin) << 2;		## ECDSA / ECC-GOST
-}
-
-
-sub keytag {
-	my $self = shift;
-
-	my $keybin = $self->keybin || return 0;
-
-	# RFC4034 Appendix B.1: most significant 16 bits of least significant 24 bits
-	return unpack 'n', substr $keybin, -3 if $self->{algorithm} == 1;
-
-	# RFC4034 Appendix B
-	my $od = length($keybin) & 1;
-	my $rd = pack "n C2 a* x$od", @{$self}{qw(flags protocol algorithm)}, $keybin;
-	my $ac = 0;
-	$ac += $_ for unpack 'n*', $rd;
-	$ac += ( $ac >> 16 );
-	return $ac & 0xFFFF;
-}
+########################################
 
 
 1;
@@ -262,7 +278,7 @@ __END__
 =head1 SYNOPSIS
 
     use Net::DNS;
-    $rr = new Net::DNS::RR('name DNSKEY flags protocol algorithm publickey');
+    $rr = Net::DNS::RR->new('name DNSKEY flags protocol algorithm publickey');
 
 =head1 DESCRIPTION
 
@@ -397,7 +413,7 @@ Package template (c)2009,2012 O.M.Kolkman and R.W.Franks.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the above copyright notice appear in all copies and that both that
+that the original copyright notices appear in all copies and that both
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
@@ -414,9 +430,11 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::RR>, RFC4034, RFC3755
+L<perl> L<Net::DNS> L<Net::DNS::RR>
+L<RFC4034|https://tools.ietf.org/html/rfc4034>
 
-L<Algorithm Numbers|http://www.iana.org/assignments/dns-sec-alg-numbers>,
 L<DNSKEY Flags|http://www.iana.org/assignments/dnskey-flags>
 
+L<Algorithm Numbers|http://www.iana.org/assignments/dns-sec-alg-numbers>
+
 =cut
