@@ -2,7 +2,7 @@ package Net::DNS::RR::SVCB;
 
 use strict;
 use warnings;
-our $VERSION = (qw$Id: SVCB.pm 1845 2021-07-14 15:30:16Z willem $)[2];
+our $VERSION = (qw$Id: SVCB.pm 1896 2023-01-30 12:59:25Z willem $)[2];
 
 use base qw(Net::DNS::RR);
 
@@ -15,7 +15,6 @@ Net::DNS::RR::SVCB - DNS SVCB resource record
 
 use integer;
 
-use Carp;
 use MIME::Base64;
 use Net::DNS::DomainName;
 use Net::DNS::RR::A;
@@ -31,12 +30,12 @@ my %keybyname = (
 	ipv4hint	  => 'key4',
 	ech		  => 'key5',
 	ipv6hint	  => 'key6',
+	dohpath		  => 'key7',				# draft-schwartz-svcb-dns
 	);
 
 
 sub _decode_rdata {			## decode rdata from wire-format octet string
-	my $self = shift;
-	my ( $data, $offset ) = @_;
+	my ( $self, $data, $offset ) = @_;
 
 	my $rdata = substr $$data, $offset, $self->{rdlength};
 	$self->{SvcPriority} = unpack( 'n', $rdata );
@@ -75,15 +74,18 @@ sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
 	my $priority = $self->{SvcPriority};
+	my $target   = $self->{TargetName}->string;
 	my $params   = $self->{SvcParams} || [];
-	return ( $priority, $self->{TargetName}->string ) unless scalar @$params;
+	return ( $priority, $target ) unless scalar @$params;
 
+	my $encode = $self->{TargetName}->encode();
+	my $length = 2 + length $encode;
+	my @target = grep {length} split /(\S{32})/, unpack 'H*', $encode;
 	my @rdata  = unpack 'H4', pack 'n', $priority;
-	my $target = $self->{TargetName}->encode();
-	my $length = 2 + length $target;
-	my @target = split /(\S{32})/, unpack 'H*', $target;
-	push @rdata, $length > 18 ? "\t; $priority\n" : (), @target;
-	push @rdata, join '', "\t; ", $self->{TargetName}->string, "\n" if $length > 3;
+	push @rdata, "\t; priority: $priority\n";
+	push @rdata, shift @target;
+	push @rdata, join '', "\t; target: ", substr( $target, 0, 50 ), "\n";
+	push @rdata, @target;
 
 	my @params = @$params;
 	while (@params) {
@@ -91,7 +93,7 @@ sub _format_rdata {			## format rdata portion of RR string.
 		my $val = shift @params;
 		push @rdata, "\n";
 		push @rdata, "; key$key=...\n" if $key > 15;
-		push @rdata, unpack 'H4H4', pack( 'n2', $key, length $val );
+		push @rdata, unpack 'H4H4',    pack( 'n2', $key, length $val );
 		push @rdata, split /(\S{32})/, unpack 'H*', $val;
 		$length += 4 + length $val;
 	}
@@ -100,24 +102,23 @@ sub _format_rdata {			## format rdata portion of RR string.
 
 
 sub _parse_rdata {			## populate RR from rdata in argument list
-	my $self = shift;
+	my ( $self, @argument ) = @_;
 
-	$self->svcpriority(shift);
-	$self->targetname(shift);
+	$self->svcpriority( shift @argument );
+	$self->targetname( shift @argument );
 
 	local $SIG{__WARN__} = sub { die @_ };
-	while ( my $svcparam = shift ) {
+	while ( my $svcparam = shift @argument ) {
 		for ($svcparam) {
 			my @value;
 			if (/^key\d+=(.*)$/i) {
-				push @value, length($1) ? $1 : shift;
-			} elsif (/=(.*)$/) {
-				local $_ = length($1) ? $1 : shift;
-				s/^(["'])(.*)\1$/$2/;		# strip paired quotes
-				s/\\,/\\044/g;			# disguise escaped comma
+				push @value, length($1) ? $1 : shift @argument;
+			} elsif (/^[^=]+=(.*)$/) {
+				local $_ = length($1) ? $1 : shift @argument;
+				s/^"([^"]*)"$/$1/;		# strip enclosing quotes
 				push @value, split /,/;
 			} else {
-				push @value, '' unless $keybyname{lc $_};    # empty keyNNN
+				push @value, '' unless $keybyname{lc $_};    # empty | Boolean
 			}
 
 			s/[-]/_/g;				# extract identifier
@@ -132,19 +133,20 @@ sub _parse_rdata {			## populate RR from rdata in argument list
 sub _post_parse {			## parser post processing
 	my $self = shift;
 
-	my $params = $self->{SvcParams} || return;
-	my %params = @$params;
+	my $paramref = $self->{SvcParams} || [];
+	my %svcparam = scalar(@$paramref) ? @$paramref : return;
+
 	$self->key0(undef);					# ruse to force sorting of SvcParams
-	if ( defined $params{0} ) {
+	if ( defined $svcparam{0} ) {
 		my %unique;
-		foreach ( grep { !$unique{$_}++ } unpack 'n*', $params{0} ) {
-			croak( $self->type . qq[: unexpected "key0" in mandatory list] ) if $unique{0};
-			croak( $self->type . qq[: duplicate "key$_" in mandatory list] ) if --$unique{$_};
-			croak( $self->type . qq[: mandatory "key$_" not present] ) unless defined $params{$_};
+		foreach ( grep { !$unique{$_}++ } unpack 'n*', $svcparam{0} ) {
+			die( $self->type . qq[: unexpected "key0" in mandatory list] ) if $unique{0};
+			die( $self->type . qq[: duplicate "key$_" in mandatory list] ) if --$unique{$_};
+			die( $self->type . qq[: mandatory "key$_" not present] ) unless defined $svcparam{$_};
 		}
 		$self->mandatory( keys %unique );		# restore mandatory key list
 	}
-	croak( $self->type . qq[: expected alpn="..." not present] ) if defined( $params{2} ) and !$params{1};
+	die( $self->type . qq[: expected alpn="..." not present] ) if defined( $svcparam{2} ) && !$svcparam{1};
 	return;
 }
 
@@ -158,17 +160,16 @@ sub _defaults {				## specify RR attribute default values
 
 
 sub svcpriority {
-	my $self = shift;					# uncoverable pod
-
-	$self->{SvcPriority} = 0 + shift if scalar @_;
+	my ( $self, @value ) = @_;				# uncoverable pod
+	for (@value) { $self->{SvcPriority} = 0 + $_ }
 	return $self->{SvcPriority} || 0;
 }
 
 
 sub targetname {
-	my $self = shift;					# uncoverable pod
+	my ( $self, @value ) = @_;				# uncoverable pod
 
-	$self->{TargetName} = Net::DNS::DomainName->new(shift) if scalar @_;
+	for (@value) { $self->{TargetName} = Net::DNS::DomainName->new($_) }
 
 	my $target = $self->{TargetName} ? $self->{TargetName}->name : return;
 	return $target unless $self->{SvcPriority};
@@ -176,107 +177,121 @@ sub targetname {
 }
 
 
-########################################
-
-
-sub _presentation {			## render octet string(s) in presentation format
-	return () unless scalar @_;
-	my $raw = join '', @_;
-	my $txt = Net::DNS::Text->decode( \$raw, 0, length($raw) );
-	return map { s/ /\\032/g; s/,/\\044/g; $_ } $txt->string;
-}
-
-sub _base64 {
-	return _presentation( map { MIME::Base64::decode($_) } @_ );
-}
-
-sub _integer16 {
-	return _presentation( map { pack( 'n', $_ ) } @_ );
-}
-
-sub _ipv4 {
-	return _presentation( map { Net::DNS::RR::A::address( {}, $_ ) } @_ );
-}
-
-sub _ipv6 {
-	return _presentation( map { Net::DNS::RR::AAAA::address( {}, $_ ) } @_ );
-}
-
-sub _string {
-	return _presentation( map { Net::DNS::Text->new($_)->encode() } @_ );
-}
-
-
 sub mandatory {				## mandatory=key1,port,...
-	my $self = shift;
-	my @list = map { $keybyname{lc $_} || $_ } map { split /,/ } @_;
-	my @keys = map { /(\d+)$/ ? $1 : croak( $self->type . qq[: unexpected "$_"] ) } @list;
+	my ( $self, @value ) = @_;
+	my @list = map { $keybyname{lc $_} || $_ } map { split /,/ } @value;
+	my @keys = map { /(\d+)$/ ? $1 : die( $self->type . qq[: unexpected "$_"] ) } @list;
 	return $self->key0( _integer16( sort { $a <=> $b } @keys ) );
 }
 
 sub alpn {				## alpn=h3,h2,...
-	my $self = shift;
-
-	###	tolerate unnecessary double-escape nonsense in draft-ietf-dnsop-svcb-https	###
-	my @sanitized = map { s/\\092,/\\044/g; s/\\092\\092/\\092/g; split /,/ } join ',', @_;
-	return $self->key1( scalar(@_) ? _string(@sanitized) : () );
+	my ( $self, @value ) = @_;
+	return $self->key1( _string(@value) );
 }
 
 sub no_default_alpn {			## no-default-alpn
-	my $self = shift;					# uncoverable pod
-	return $self->key2( ( defined(wantarray) ? @_ : '' ), @_ );
+	my ( $self, @value ) = @_;				# uncoverable pod
+	return $self->key2( ( defined(wantarray) ? @value : '' ), @value );
 }
 
 sub port {				## port=1234
-	my $self = shift;
-	return $self->key3( map { _integer16($_) } @_ );
+	my ( $self, @value ) = @_;
+	return $self->key3( map { _integer16($_) } @value );
 }
 
 sub ipv4hint {				## ipv4hint=192.0.2.1,...
-	my $self = shift;
-	return $self->key4( _ipv4(@_) );
+	my ( $self, @value ) = @_;
+	return $self->key4( _ipv4(@value) );
 }
 
 sub ech {				## ech=base64string
-	my $self = shift;
-	return $self->key5( map { _base64($_) } @_ );
+	my ( $self, @value ) = @_;
+	return $self->key5( map { _base64($_) } @value );
 }
 
 sub ipv6hint {				## ipv6hint=2001:DB8::1,...
-	my $self = shift;
-	return $self->key6( _ipv6(@_) );
+	my ( $self, @value ) = @_;
+	return $self->key6( _ipv6(@value) );
+}
+
+sub dohpath {				## dohpath=/dns-query{?dns}
+	my ( $self, @value ) = @_;				# uncoverable pod
+	return $self->key7(@value);
+}
+
+
+########################################
+
+
+sub _presentation {			## render octet string(s) in presentation format
+	my @arg = @_;
+	my $raw = scalar(@arg) ? join( '', @arg ) : return ();
+	return Net::DNS::Text->decode( \$raw, 0, length($raw) )->string;
+}
+
+sub _base64 {
+	my @arg = @_;
+	return _presentation( map { MIME::Base64::decode($_) } @arg );
+}
+
+sub _integer16 {
+	my @arg = @_;
+	return _presentation( map { pack( 'n', $_ ) } @arg );
+}
+
+sub _ipv4 {
+	my @arg = @_;
+	return _presentation( map { Net::DNS::RR::A::address( {}, $_ ) } @arg );
+}
+
+sub _ipv6 {
+	my @arg = @_;
+	return _presentation( map { Net::DNS::RR::AAAA::address( {}, $_ ) } @arg );
+}
+
+sub _string {
+	my @arg = @_;
+	local $_ = join ',', @arg;				# reassemble argument string
+	s/\\,/\\044/g;						# disguise (RFC1035) escaped comma
+	die <<"QQ" if /\\092,|\\092\\092/;
+SVCB: Please use standard RFC1035 escapes\n draft-ietf-dnsop-svcb-https double-escape nonsense not implemented
+QQ
+	return _presentation( map { Net::DNS::Text->new($_)->encode() } split /,/ );
 }
 
 
 our $AUTOLOAD;
 
 sub AUTOLOAD {				## Dynamic constructor/accessor methods
-	my $self = shift;
+	my ( $self, @argument ) = @_;
 
 	my ($method) = reverse split /::/, $AUTOLOAD;
 
-	my $default = join '::', 'SUPER', $method;
-	return $self->$default(@_) unless $method =~ /^key[0]*(\d+)$/i;
+	my $super = "SUPER::$method";
+	return $self->$super(@argument) unless $method =~ /^key[0]*(\d+)$/i;
 	my $key = $1;
 
-	my $params = $self->{SvcParams} || [];
-	my %params = @$params;
+	my $paramsref = $self->{SvcParams} || [];
+	my %svcparams = @$paramsref;
 
-	if ( scalar @_ ) {
-		my $arg = shift;				# keyNN($value);
-		delete $params{$key} unless defined $arg;
-		croak( $self->type . qq[: duplicate SvcParam "key$key"] ) if defined $params{$key};
-		$params{$key} = Net::DNS::Text->new("$arg")->raw if defined $arg;
-		$self->{SvcParams} = [map { ( $_, $params{$_} ) } sort { $a <=> $b } keys %params];
-		croak( $self->type . qq[: unexpected number of values for "key$key"] ) if scalar @_;
+	if ( scalar @argument ) {
+		my $arg = shift @argument;			# keyNN($value);
+		delete $svcparams{$key} unless defined $arg;
+		die( $self->type . qq[: duplicate SvcParam "key$key"] ) if defined $svcparams{$key};
+		die( $self->type . qq[: invalid SvcParam "key$key"] )	if $key > 65534;
+		$svcparams{$key}   = Net::DNS::Text->new("$arg")->raw if defined $arg;
+		$self->{SvcParams} = [map { ( $_, $svcparams{$_} ) } sort { $a <=> $b } keys %svcparams];
+		die( $self->type . qq[: unexpected number of values for "key$key"] ) if scalar @argument;
 	} else {
-		croak( $self->type . qq[: no value specified for "key$key"] ) unless defined wantarray;
+		die( $self->type . qq[: no value specified for "key$key"] ) unless defined wantarray;
 	}
 
-	my $value = $params{$key};
+	my $value = $svcparams{$key};
 	return defined($value) ? _presentation($value) : $value;
 }
 
+########################################
+
 
 1;
 __END__
@@ -285,7 +300,7 @@ __END__
 =head1 SYNOPSIS
 
     use Net::DNS;
-    $rr = Net::DNS::RR->new('name HTTPS SvcPriority TargetName alpn=h3,...');
+    $rr = Net::DNS::RR->new('name HTTPS SvcPriority TargetName SvcParams');
 
 =head1 DESCRIPTION
 
@@ -329,7 +344,7 @@ owner name of this record must be used as the effective TargetName.
 
 =head2 mandatory, alpn, no-default-alpn, port, ipv4hint, ech, ipv6hint
 
-    $rr = Net::DNS::RR->new( 'svc.example. SVCB 1 svc.example. port=1234' );
+    $rr = Net::DNS::RR->new( 'svcb.example. SVCB 1 svcb.example. port=1234' );
 
     $rr->port(1234);
     $string = $rr->port();	# \004\210
@@ -347,7 +362,7 @@ The behaviour with undefined arguments is not specified.
     $rr->keyNN( undef );
 
 Generic constructor and accessor methods for SvcParams.
-The key index NN is a decimal integer in the range 0 .. 65534.
+The key index NN is a decimal integer in the range 0 .. 65535.
 The method argument and returned value are both presentation format strings.
 The method returns the undefined value if the key is not present.
 The specified key will be deleted if the value is undefined.
@@ -355,7 +370,7 @@ The specified key will be deleted if the value is undefined.
 
 =head1 COPYRIGHT
 
-Copyright (c)2020-2021 Dick Franks. 
+Copyright (c)2020-2022 Dick Franks. 
 
 All rights reserved.
 
@@ -366,7 +381,7 @@ Package template (c)2009,2012 O.M.Kolkman and R.W.Franks.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the above copyright notice appear in all copies and that both that
+that the original copyright notices appear in all copies and that both
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
@@ -383,6 +398,10 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::RR>, draft-ietf-dnsop-svcb-https
+L<perl> L<Net::DNS> L<Net::DNS::RR>
+draft-ietf-dnsop-svcb-https,
+L<RFC1035|https://tools.ietf.org/html/rfc1035>
+
+L<Service Parameter Keys|https://www.iana.org/assignments/dns-svcb>
 
 =cut
